package com.bureau.network

import android.net.Network
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.util.concurrent.ConcurrentHashMap
import java.util.concurrent.TimeUnit
import java.util.concurrent.atomic.AtomicReference

object RetroFitInstance {
    private const val BASE_URl_STG = "https://api.overwatch.stg.bureau.id/"
    private const val BASE_URl_PROD = "https://api.overwatch.bureau.id/"
    private const val BASE_URL_MOONSENSE = "https://api.moonsense.cloud/"
    private val retrofitMap: ConcurrentHashMap<String, AtomicReference<Retrofit>> =
        ConcurrentHashMap()
    fun getRetrofitInstance(
        baseUrl: String,
        timeOutInMs: Int = 30000,
        network: Network? = null
    ): Retrofit {
        return try {
            if (!retrofitMap.containsKey(baseUrl)) {
                val logging = HttpLoggingInterceptor()
                logging.level =
                    if (BuildConfig.DEBUG) HttpLoggingInterceptor.Level.BODY else HttpLoggingInterceptor.Level.NONE
                val okHttpClientBuilder = OkHttpClient.Builder().addInterceptor(logging)
                    .readTimeout(timeOutInMs.toLong(), TimeUnit.MILLISECONDS)
                    .connectTimeout(timeOutInMs.toLong(), TimeUnit.MILLISECONDS)
                    .writeTimeout(timeOutInMs.toLong(), TimeUnit.MILLISECONDS)

                val okHttpClient =
                    if (network != null) {
                        okHttpClientBuilder.socketFactory(network.socketFactory)
                            .build()
                    } else {
                        okHttpClientBuilder.build()
                    }

                retrofitMap[baseUrl] = AtomicReference(
                    Retrofit.Builder()
                        .baseUrl(baseUrl)
                        .addConverterFactory(GsonConverterFactory.create())
                        .client(okHttpClient)
                        .build()
                )
            }
            retrofitMap[baseUrl]!!.get()
        } catch (exception: Exception) {
            throw IllegalArgumentException("Invalid base url hence can't create rest client $baseUrl")
        }
    }
}
